@@ -1,3 +1,5 @@
+{-# LANGUAGE CPP #-}
+
 {-|
   IP routing table is a tree of 'AddrRange'
   to search one of them on the longest
@@ -7,15 +9,19 @@
 -}
 module Data.IP.RouteTable.Internal where
 
-import Control.Applicative ((<$>))
+#if __GLASGOW_HASKELL__ < 709
+import Control.Applicative ((<$>),(<*>),pure)
+#endif
 import Control.Monad
 import Data.Bits
+import Data.Foldable (Foldable(..))
 import Data.IP.Addr
 import Data.IP.Op
 import Data.IP.Range
 import Data.IntMap (IntMap, (!))
 import qualified Data.IntMap as IM (fromList)
-import Data.List (foldl')
+import Data.Monoid
+import Data.Traversable
 import Data.Word
 import Prelude hiding (lookup)
 
@@ -101,6 +107,14 @@ instance Functor (IPRTable k) where
     fmap _ Nil = Nil
     fmap f (Node r a mv b1 b2) = Node r a (f <$> mv) (fmap f b1) (fmap f b2)
 
+instance Foldable (IPRTable k) where
+    foldMap _ Nil = mempty
+    foldMap f (Node _ _ mv b1 b2) = foldMap f mv <> foldMap f b1 <> foldMap f b2
+
+instance Traversable (IPRTable k) where
+    traverse _ Nil = pure Nil
+    traverse f (Node r a mv b1 b2) = Node r a <$> traverse f mv <*> traverse f b1 <*> traverse f b2
+
 ----------------------------------------------------------------
 
 {-|
